/*************************************************************************
 *
 *  $RCSfile: parser.hxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: hr $ $Date: 2003/03/27 11:07:06 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef SETUP_PARSER_HXX
#define SETUP_PARSER_HXX

// Parser: rekursiver Abstieg

#if SUPD < 364

#else
#ifndef _STREAM_HXX //autogen
#include <tools/stream.hxx>
#endif
#endif

#include <setup2/lexem.hxx>

class SiScanner;

class SiParser
{
	BOOL 	   m_bHaveApp;
	BOOL 	   m_bHaveVcl;
// properties
	SiScanner *m_pScanner;
	BOOL       m_bSyntaxError;

// operations
	BOOL IsDeclarator(SiLexem const&) const;
	BOOL ParseDeclaration();
	BOOL ParsePropertyList();
	BOOL ParseProperty();
	BOOL ParseValueList();

// overridables
protected:
	virtual void OnDeclaration    (SiLexemType,SiIdentifier const&)		{};
	virtual void OnDeclarationEnd ()                 					{};
	virtual void OnProperty       (ByteString const&, int nLanguage)	{};
	virtual void OnSingleValue    (sal_Int32)  							{};
	virtual void OnSingleValue    (sal_uInt32)  		 				{};
	virtual void OnSingleValue    (ByteString const&)    			   	{};
	virtual void OnSingleValue    (SiIdentifier const&)					{};
	virtual void OnListValue      (sal_Int32)		  					{};
	virtual void OnListValue      (ByteString const&)					{};
	virtual void OnListValue      (SiIdentifier const&)					{};

	enum ExpectedLexem
	{
		EXP_DECLARATOR,
		EXP_END,
		EXP_POPEN,
		EXP_PCLOSE,
		EXP_IDENTIFIER,
		EXP_ASSIGN,
		EXP_VALUE,          // rechte Seite eines Properties
		EXP_SINGLEVALUE,    // int | string | id
		EXP_INTEGER,
		EXP_SEMICOLON
	};

	void SyntaxError(SiLexem const&, ExpectedLexem);

	virtual void DisplayError(
					int nLine,
					ByteString const& ErrorType,
					ByteString const& Message) const;

	virtual void ResetError();

public:
	SiParser(SvStream &);
	~SiParser();

	void		NoApplication()		   				{ m_bHaveApp = FALSE; }
	BOOL		IsApplication() const  				{ return m_bHaveApp; }

    void		NoVCL()                             { m_bHaveVcl = FALSE; }
	BOOL		HasVCL() const                      { return m_bHaveVcl; }

	BOOL		ParseScript();
	void		Recover();
	BOOL		Continue();
	BOOL		IsSyntaxError() const				{ return m_bSyntaxError; }
	int 		GetLine() const;
};

#endif // _PARSER_HXX

