/*************************************************************************
 *
 *  $RCSfile: internaloptions.hxx,v $
 *
 *  $Revision: 1.6 $
 *
 *  last change: $Author: cd $ $Date: 2001/08/21 16:14:30 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef INCLUDED_SVTOOLS_INTERNALOPTIONS_HXX
#define INCLUDED_SVTOOLS_INTERNALOPTIONS_HXX

//_________________________________________________________________________________________________________________
//	includes
//_________________________________________________________________________________________________________________

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

//_________________________________________________________________________________________________________________
//	namespaces!
//_________________________________________________________________________________________________________________

#define	MUTEX			::osl::Mutex
#define	OUSTRING		::rtl::OUString

//_________________________________________________________________________________________________________________
//	forward declarations
//_________________________________________________________________________________________________________________

/*-************************************************************************************************************//**
	@short			forward declaration to our private date container implementation
	@descr			We use these class as internal member to support small memory requirements.
					You can create the container if it is neccessary. The class which use these mechanism
					is faster and smaller then a complete implementation!
*//*-*************************************************************************************************************/

class SvtInternalOptions_Impl;

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

/*-************************************************************************************************************//**
	@short			collect informations about internal features
	@descr          -

	@implements		-
	@base			-

	@ATTENTION		This class is partially threadsafe.

	@devstatus		ready to use
*//*-*************************************************************************************************************/

class SvtInternalOptions
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard constructor and destructor
			@descr		This will initialize an instance with default values.
						We implement these class with a refcount mechanism! Every instance of this class increase it
						at create and decrease it at delete time - but all instances use the same data container!
						He is implemented as a static member ...

			@seealso	member m_nRefCount
			@seealso	member m_pDataContainer

			@param		-
			@return		-

			@onerror	-
		*//*-*****************************************************************************************************/

         SvtInternalOptions();
        ~SvtInternalOptions();

		//---------------------------------------------------------------------------------------------------------
		//	interface
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		interface methods to get and set value of config key "org.openoffice.Office.Common/Internal/..."
			@descr      These options describe internal states to enable/disable features of installed office.
						The values are fixed at runtime - and implemented as readonly!

						SlotCFGEnabled()	=>  If this option is set (true), StarOffice is searching for the slot.cfg.
												If the slot.cfg cannot be found, the start is aborted.
												If this option is not set (false), the slot.cfg must not be available,
												otherwise the start is also aborted.

						CrashMailEnabled()	=>  Crash-Mail-Feature to document program crashes. After a crash,
												an e-mail with information about the system used is generated
												automatically when starting StarOffice.

			@seealso	configuration package "org.openoffice.Office.Common/Internal"
		*//*-*****************************************************************************************************/

		sal_Bool	SlotCFGEnabled		() const;
		sal_Bool	CrashMailEnabled	() const;
        sal_Bool	MailUIEnabled      () const;
        sal_Bool	IsRemoveMenuEntryClose() const;
        sal_Bool	IsRemoveMenuEntryBackToWebtop() const;
        sal_Bool	IsRemoveMenuEntryNewWebtop() const;
        sal_Bool	IsRemoveMenuEntryLogout() const;

		OUSTRING	GetCurrentTempURL() const;
		void		SetCurrentTempURL( const OUSTRING& aNewCurrentTempURL );

		/*-****************************************************************************************************//**
			@short		recovery list for our CrashGuard
			@descr      If office crash - we must save list of current open files.
						These list will read and delete at next start. We implement it as a stack.
						To push something works everytime - pop values return 3 empty strings if stack empty!

			@seealso	configuration package "org.openoffice.Office.Common/Internal"
		*//*-*****************************************************************************************************/

		void PushRecoveryItem(	const	OUSTRING&	sURL		,
								const	OUSTRING&	sFilter		,
								const	OUSTRING&	sTempName	);
		void PopRecoveryItem(			OUSTRING&	sURL		,
										OUSTRING&	sFilter		,
										OUSTRING&	sTempName	);
		sal_Bool IsRecoveryListEmpty() const;

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		return a reference to a static mutex
			@descr		These class is partially threadsafe (for de-/initialization only).
						All access methods are'nt safe!
						We create a static mutex only for one ime and use at different times.

			@seealso	-

			@param		-
			@return		A reference to a static mutex member.

			@onerror	-
		*//*-*****************************************************************************************************/

		static MUTEX& GetOwnStaticMutex();

	//-------------------------------------------------------------------------------------------------------------
	//	private member
	//-------------------------------------------------------------------------------------------------------------

	private:

		/*Attention

			Don't initialize these static member in these header!
			a) Double dfined symbols will be detected ...
			b) and unresolved externals exist at linking time.
			Do it in your source only.
		 */

    	static SvtInternalOptions_Impl*	m_pDataContainer	;	/// impl. data container as dynamic pointer for smaller memory requirements!
		static sal_Int32				m_nRefCount			;	/// internal ref count mechanism

};		// class SvtInternalOptions

#endif	// #ifndef INCLUDED_SVTOOLS_INTERNALOPTIONS_HXX
