/*************************************************************************
 *
 *  $RCSfile: ctrl.hxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: pl $ $Date: 2002/05/16 11:52:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_CTRL_HXX
#define _SV_CTRL_HXX

#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_WINDOW_HXX
#include <vcl/window.hxx>
#endif

// forward
namespace vcl { struct ControlLayoutData; }

// -----------
// - Control -
// -----------

class Control : public Window
{
protected:
	mutable vcl::ControlLayoutData*			mpLayoutData;
private:
	long										mnDummy1;
	BOOL										mbHasFocus;
	BOOL										mbDummy1;
	BOOL										mbDummy2;
	BOOL										mbDummy3;
	Link										maGetFocusHdl;
	Link										maLoseFocusHdl;

#ifdef _SV_CTRL_CXX
	void			ImplInitData();
public:
    void			ImplSubControlLayoutChanged() const;
#endif
protected:
					Control( WindowType nType );
    virtual void  FillLayoutData() const;

    // helper method for composite controls
    void			AppendLayoutData( const Control& rSubControl ) const;
public:
					Control( Window* pParent, WinBits nWinStyle = 0 );
					Control( Window* pParent, const ResId& rResId );
    				~Control();

	virtual void	GetFocus();
	virtual void	LoseFocus();
	virtual long	Notify( NotifyEvent& rNEvt );
    virtual void	StateChanged( StateChangedType nStateChange );
    virtual void  Resize();

    // invalidates layout data
    virtual void	SetText( const String& rStr );
    // gets the displayed text
    virtual String GetDisplayText() const;
    // returns the bounding box for the character at index nIndex (in control coordinates)
    Rectangle GetCharacterBounds( long nIndex ) const;
    // returns the character index for corresponding to rPoint (in control coordinates)
    // -1 is returned if no character is at that point
    long GetIndexForPoint( const Point& rPoint ) const;
    // returns the number of lines in the result of GetDisplayText()
    long GetLineCount() const;
    // returns the interval [start,end] of line nLine
    // returns [-1,-1] for an invalid line
    Pair GetLineStartEnd( long nLine ) const;

	void			SetGetFocusHdl( const Link& rLink ) { maGetFocusHdl = rLink; }
	const Link& 	GetGetFocusHdl() const	 { return maGetFocusHdl; }
	void			SetLoseFocusHdl( const Link& rLink ) { maLoseFocusHdl = rLink; }
	const Link& 	GetLoseFocusHdl() const { return maLoseFocusHdl; }

    void			SetLayoutDataParent( const Control* pParent ) const;
};

#endif	// _SV_CTRL_HXX
