/*************************************************************************
 *
 *  $RCSfile: octree.hxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: hr $ $Date: 2000/12/07 13:37:15 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_OCTREE_HXX
#define _SV_OCTREE_HXX

#ifndef _SV_SALBTYPE_HXX
#include <vcl/salbtype.hxx>
#endif

// -----------
// - Defines -
// -----------

#define OCTREE_BITS		5
#define OCTREE_BITS_1	10

// --------------
// - OctreeNode -
// --------------

typedef struct OctreeNode
{
	ULONG		nCount;
	ULONG		nRed;
	ULONG		nGreen;
	ULONG		nBlue;
	OctreeNode* pChild[ 8 ];
	OctreeNode*	pNext;
	OctreeNode*	pNextInCache;
	USHORT		nPalIndex;
	BOOL		bLeaf;
} NODE;

typedef NODE*		PNODE;
typedef PNODE*		PPNODE;

// ----------
// - Octree -
// ----------

class ImpNodeCache;
class BitmapReadAccess;

class Octree
{
private:

	BitmapPalette				aPal;
	ULONG						nMax;
	ULONG						nLeafCount;
	ULONG						nLevel;
	PNODE						pTree;
	PNODE						pReduce[ OCTREE_BITS + 1 ];
	BitmapColor*				pColor;
	ImpNodeCache*				pNodeCache;
	const BitmapReadAccess*		pAcc;
	USHORT						nPalIndex;

								Octree() {};

	void						CreatePalette( PNODE pNode );
	void						GetPalIndex( PNODE pNode );

	void						ImplCreateOctree();
	void						ImplDeleteOctree( PPNODE ppNode );
	void						ImplAdd( PPNODE ppNode );
	void						ImplReduce();

public:							

								Octree( const BitmapReadAccess& rReadAcc, ULONG nColors );
								Octree( ULONG nColors );
								~Octree();

	void						AddColor( const BitmapColor& rColor );

	inline const BitmapPalette&	GetPalette();
	inline USHORT				GetBestPaletteIndex( const BitmapColor& rColor );
};

// ------------------------------------------------------------------------

inline const BitmapPalette& Octree::GetPalette()
{
	aPal.SetEntryCount( (USHORT) nLeafCount );
	nPalIndex = 0;
	CreatePalette( pTree );
	return aPal;
}

// ------------------------------------------------------------------------

inline USHORT Octree::GetBestPaletteIndex( const BitmapColor& rColor )
{
	pColor = &(BitmapColor&) rColor;
	nPalIndex = 65535;
	nLevel = 0L;
	GetPalIndex( pTree );
	return nPalIndex;
}

// -------------------
// - InverseColorMap -
// -------------------

class InverseColorMap
{
private:

	BYTE*				pBuffer;
	BYTE*				pMap;
	const ULONG			nBits;

#if 0 // _SOLAR__PRIVATE 

	void				ImplCreateBuffers( const ULONG nMax );

#endif // __PRIVATE

public:

						InverseColorMap( const BitmapPalette& rPal );
						~InverseColorMap();

	inline USHORT		GetBestPaletteIndex( const BitmapColor& rColor );
};

// ------------------------------------------------------------------------

inline USHORT InverseColorMap::GetBestPaletteIndex( const BitmapColor& rColor )
{
	return pMap[ ( ( (ULONG) rColor.GetRed() >> nBits ) << OCTREE_BITS_1 ) | 
				 ( ( (ULONG) rColor.GetGreen() >> nBits ) << OCTREE_BITS ) | 
				 ( (ULONG) rColor.GetBlue() >> nBits ) ];
}

#endif // _SV_OCTREE_HXX
