/*************************************************************************
 *
 *  $RCSfile: sound.hxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: ka $ $Date: 2001/06/13 10:52:47 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SV_SOUND_HXX
#define _SV_SOUND_HXX

#ifndef _SV_SNDSTYLE_HXX
#include <vcl/sndstyle.hxx>
#endif

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _TIME_HXX
#include <tools/time.hxx>
#endif
#ifndef _SV_SALSTYPE_HXX
#include <vcl/salstype.hxx>
#endif

class Window;
class Timer;
class SalSound;
class RMSound;

// ---------
// - Sound -
// ---------

class Sound
{
private:

#ifndef REMOTE_APPSERVER
	SalSound*			mpSound;
#else
	RMSound*			mpSound;
#endif

	XubString			maSoundName;
	Window* 			mpWindow;
	BYTE*				mpSoundData;
	ULONG				mnDataLen;
	ULONG				mnSoundLen;
	ULONG				mnStartTime;
	ULONG				mnPlayTime;
	ULONG				mnErrorCode;
	SoundNotification	meNotification;
	BOOL				mbPlaying;
	BOOL				mbLoopMode;
    Link                maNotifyHdl;
	void*				mpExtraData;

private:

                        Sound( const Sound& rSound );
    Sound&              operator=( const Sound& rSound );

#if 0 // _SOLAR__PRIVATE
public:
	void				ImplNotify( SoundNotification eNotification, ULONG nError );
#endif // __PRIVATE

public:
                        Sound( Window* pWindow = NULL );
                        ~Sound();

    virtual void        Notify();

	void				SetWindow( Window* pWindow ) { mpWindow = pWindow; }
	Window* 			GetWindow() const { return mpWindow; }
	BOOL				SetSoundName( const XubString& rSoundName );
	const XubString& 	GetSoundName() const { return maSoundName; }
	BOOL				SetSoundData( const BYTE* pSoundData, ULONG nDataLen );
	BYTE*				GetSoundData() const { return mpSoundData; }
	ULONG				GetSoundLen() const { return mnSoundLen; }

    void				Play();
	void				Stop();
	void				Pause();
    BOOL                IsPlaying() const { return mbPlaying; }

	void				SetStartTime( ULONG nStartTime = 0 );
	ULONG				GetStartTime() const { return mnStartTime; }
	void				SetPlayTime( ULONG nPlayTime = SOUND_PLAYALL );
	ULONG				GetPlayTime() const { return mnPlayTime; }
	void				SetLoopMode( BOOL bLoop );
	BOOL				IsLoopMode() const { return mbLoopMode; }

    void                ClearError();
    ULONG               GetLastError() const { return mnErrorCode; }
    SoundNotification   GetNotification() const { return meNotification; }

    void                SetNotifyHdl( const Link& rLink ) { maNotifyHdl = rLink; }
    const Link&         GetNotifyHdl() const { return maNotifyHdl; }

    static void         Beep( SoundType eType = SOUND_DEFAULT, Window* pWindow = NULL );

	static void			SetSoundPath( const XubString& rSoundPath );
	static const XubString&	GetSoundPath();

    static BOOL         IsSoundFile( const XubString& rSoundPath );
};

#endif // _SV_SOUND_HXX

